@@ -12,7 +12,7 @@ function( rates,
           a.lim = range( age, na.rm=TRUE ) + c(0,diff( range( age ) )/30),
           p.lim = range( per, na.rm=TRUE ) + c(0,diff( range( age ) )/30),
           c.lim = NULL, 
-           ylim = range( rates[rates>0] ),
+           ylim = range( rates[rates>0], na.rm=TRUE ),
              at = NULL,
          labels = paste( at ),
           a.lab = "Age at diagnosis",
